use std::sync::Arc;

use tokio::sync::Mutex;
use tonic::transport::{Channel, ClientTlsConfig};

use crate::grpc::gen::flux_releaser_client::FluxReleaserClient;

pub type FluxReleaserGrpcClient = Arc<Mutex<FluxReleaserClient<Channel>>>;

pub async fn new_client(registry: impl Into<String>) -> anyhow::Result<FluxReleaserGrpcClient> {
    let registry: String = registry.into();

    let channel = if registry.starts_with("https") {
        let mut tls = ClientTlsConfig::new();
        tls = tls.domain_name(&registry);

        Channel::from_shared(registry)?
            .tls_config(tls)?
            .connect()
            .await?
    } else {
        Channel::from_shared(registry)?.connect().await?
    };

    let client = FluxReleaserClient::new(channel);

    Ok(Arc::new(Mutex::new(client)))
}
